% Script to calculate the property adjustments for each cruise leg that
% maximizes the consisteny among cruise legs at cross overs. 
% Input is cross-over offsets and associated uncertainties
% This script also uses pre-determined weights for the WDLSQ solution
%
% This script is for the 7 variables: sal, tco2 alk, nitrate, phosphate,
% silicate and oxygen only

% OUTPUT:   plot of crossover residuals and uncertainties
%           plot of cruise adjustments and uncertainties
%           Table of cruise adjustments and uncertainties

% INPUT:    Data has to be in a certain format, see xover_loop.m


% Ref: Johnson, Robbins and Hufford 2001, Journal of Atmospheric and Oceanic Technology.
% Volume 18 (1234-1244)
% Toste Tanhua 2009.06.05

clear all; close all;

% ===========================================
% USER INPUT
% =========================================
addormul=1;     % 1 for additative, 2 for multiplicative

% THIS SECTION TO DEINFE WHICH INPUT FILES TO RUN
if addormul == 1, % For additative offsets
    load crossover_add;  region='North Atlantic';
end

if addormul == 2, % For multiplicative offsets
    load crossover_mult;  region='North Atlantic';
end


% Activate this part if the time between a two cruises in a crossover (dely) shall
% influence its weight in the inversion (i.e. increase the standard
% deviation for large dely)
% assume a linear decline in significance as delta-year increases.
% "timefactor" is multiplied with the standard deviation to make a Xover
% result less significant. 
timefactor=ones(length(dely),1);
for i=1:length(dely),
    if abs(dely(i))<900, %Time series will have dely=1000 (i.e. indefinite dely)
        timefactor(i)=dely(i)*0.1+1; % Change here to increase the sensitivity to time and latitude
    end
end

% The cruise has to be renumered to be consecutive
% First save the orignal cruise numbers
cr1org=cr1; cr2org=cr2;
uc=unique([cr1 cr2]);
VIK=[];
for i=1:length(uc),
    F=find(cr1 == uc(i)); cr1(F)=i;
    F=find(cr2 == uc(i)); cr2(F)=i;
    % Reduce the weight matrix accordingly
    F=find(vikter(:,1) == uc(i)); VV=vikter(i,:);
    VIK=[VIK;VV];
end
vikter=VIK(:,2:length(variables)+1);

% ===========================================
% END USER INPUT
% ===========================================

%cruisenomax=max(max([cr1 cr2]));
% Increase the uncertainty in the adjustment with timefactor
TT=[];
for i=1:length(variables),
    tt=orgstd(:,i).*timefactor;
    TT=[TT , tt];
end
orgstd=TT;

% Create matrix "G", the model matrix (a matrix of "0", "1" and "-1") t
for j=1:(length(cr2));
    G(j,cr1(j))=1;
    G(j,cr2(j))=-1;
end

P=[1:1:max([cr1 ; cr2])]';  % To make a list of cruises used ("acronym")
%=================================
% THESE ARE THE INVERSIONS
% Calculate SLSQ, WLSQ and WDLSQ, i.e Simple Least Square, Weighted Least
% Squares and Weighted Damped Least Squares
 SLSQ=[]; WLSQ=[]; WDLSQ=[]; covSLSQ=[]; covWLSQ=[]; covWDLSQ=[];
 for i =1:length(variables) ;  % loop through the variables
     % First cut out cross overs with NaN for this variable
     d=orginal(:,i);    % d is offset values for each cross over for variable i (data vector)
     f=orgstd(:,i);     % f is the variance vector, i.e. the std of the Xovers
     % Don't allow the std to be 0
     f=f+0.0000001;
     F=find(~isnan(d) & ~isnan(f)); d=d(F); GG=G(F,:); f=f(F); % cut out not valid Xovers
     dstd=std(d);  % 
          
     W=diag(f.^-1);             % Data covariance matrix
     E=diag(vikter(:,i).^2);    % model covariance matrix 
     
     % Find cruises with unsufficient data for a particular variable to do an adjustment
     U=unique([cr1(F) cr2(F)]);  T=setdiff(P,U);
     
     % SLSQ calculations
     SLSQtemp=pinv(GG)*d; 
     temp=cov(-GG*SLSQtemp+d)*pinv(GG'*GG);    covSLSQtemp=real(sqrt(diag(temp)));
     
     % WLSQ calculations
     W=diag(f.^-1);         % Data covariance matrix
     WLSQtemp=pinv(W*GG)*(W*d);        
     temp=pinv(GG'*W^2*GG);      covWLSQtemp=real(sqrt(diag(temp)));
     
     % WDLSQ calculations
     W=diag(f.^2);         % Data covariance matrix
     WDLSQtemp=E*GG'*(inv(GG*E*GG'+W)*d);  
     temp=E-E*GG'*inv(GG*E*GG'+W)*GG*E;      covWDLSQtemp=real(sqrt(diag(temp)));
       
     % Add the missing xovers, and save the solutions for all variables
     SLSQtemp(T)=NaN;           WLSQtemp(T)=NaN;        WDLSQtemp(T)=NaN;
     SLSQ=[SLSQ , SLSQtemp];    WLSQ=[WLSQ , WLSQtemp]; WDLSQ=[WDLSQ , WDLSQtemp];
     
     covSLSQtemp(T)=NaN;                covWLSQtemp(T)=NaN;                 covWDLSQtemp(T)=NaN;  
     covSLSQ=[covSLSQ , covSLSQtemp];   covWLSQ=[covWLSQ , covWLSQtemp];    covWDLSQ=[covWDLSQ , covWDLSQtemp];
 end

% To do the sign convention right so that the result is the adjustment that
% has to be applied (i.e. added or multiplied with).
  SLSQ=-SLSQ; WLSQ=-WLSQ; WDLSQ=-WDLSQ;

 % =================================
 % CHECK AND DIPLAY THE RESULTS
  if addormul > 1,
     orginal=orginal+1;  % Add unity to the multiplicative
  end
 
 % First check how the Xover offsets changed
 VV=[];   LSQ=SLSQ;   % For SLSQ results
 for i=1:length(variables),  % Do for each variable
     XX=[];
     for j=1:length(cr1),  % Do for each cross over
         if addormul == 1, % For additative corrections
             t=LSQ(cr1(j),i)-LSQ(cr2(j),i);  % This is the correction to Xover j and variable i
             X=orginal(j,(i))+t;  % This is the Xover offset after the correction has been applied
         else  % For multiplicative corrections
            t1=(LSQ(cr1(j),i)+1); t2=(LSQ(cr2(j),i)+1);  % This is the correction to Xover j and variable i
            X=orginal(j,(i))*t1/t2;
         end
         XX=[XX;X];
     end
     VV=[VV , XX];
 end
 SLSQcorr=VV;  % This is the new Xover differences with SLSQ corrections applied
 
 VV=[];   LSQ=WLSQ;   % For WLSQ results
 for i=1:length(variables),  % Do for each variable
     XX=[];
     for j=1:length(cr1),  % Do for each cross over
         if addormul == 1, % For additative corrections
             t=LSQ(cr1(j),i)-LSQ(cr2(j),i);  % This is the correction to Xover j and variable i
             X=orginal(j,(i))+t;  % This is the Xover offset after the correction has been applied
         else  % For multiplicative corrections
            t1=(LSQ(cr1(j),i)+1); t2=(LSQ(cr2(j),i)+1);  % This is the correction to Xover j and variable i
            X=orginal(j,(i))*t1/t2;
         end
         XX=[XX;X];
     end
     VV=[VV , XX];
 end
 WLSQcorr=VV;  % This is the new Xover differences with WLSQ corrections applied
 
 VV=[];   LSQ=WDLSQ;   % For WDLSQ results
 for i=1:length(variables),  % Do for each variable
     XX=[];
     for j=1:length(cr1),  % Do for each cross over
         if addormul == 1, % For additative corrections
             t=LSQ(cr1(j),i)-LSQ(cr2(j),i);  % This is the correction to Xover j and variable i
             X=orginal(j,(i))+t;  % This is the Xover offset after the correction has been applied
         else  % For multiplicative corrections
            t1=(LSQ(cr1(j),i)+1); t2=(LSQ(cr2(j),i)+1);  % This is the correction to Xover j and variable i
            X=orginal(j,(i))*t1/t2;
         end
         XX=[XX;X];
     end
     VV=[VV , XX];
 end
 WDLSQcorr=VV;  % This is the new Xover differences with WDLSQ corrections applied
 
 
% =========================================================================
% PLOTS
% =========================================================================
te=size(SLSQ); va=te(2); % Numbers of variables

posfig(1);
for i=1:va, % Numbers of variables done
       
    % Plots for the original data
    subplot(va,4,(i-1)*4+1);
    V1=orginal(:,i); E1=orgstd(:,i);
    [V1,I]=sort(V1);
    errorbar(V1,E1(I),'k.-'); hold on
    plot(V1,'.r'); 
    L=find(~isnan(V1)); 
    if i == 1,
       title('Initial');
    end
    if isempty(L),
        set(gca,'xlim',[0 1]);
    else
        set(gca,'xlim',[0 length(L)]);
    end
    if addormul == 1,
        yl=[100 30 30 0.2];  % Set the Y-limits
        set(gca,'ylim',[-yl(i) yl(i)]); 
        plot(xlim,[0 0],'r-');
        F=find((abs(V1)-E1(I)) < 0);  T=find(~isnan(V1));  andel=length(F)/length(T)*100;
        mm=sprintf('%2.0f',andel);   text(length(L)/20,yl(i)*7/8,mm,'color','r');
        ylabel(variables(i));
    elseif addormul == 2,
        yl=[0.4 0.4 0.4 0.1 1.0]; % Set the Y-limits
        set(gca,'ylim',[1-yl(i) 1+yl(i)]);
        plot(xlim,[1 1],'r-');
        F=find((abs(V1-1)-E1(I)) < 0);  T=find(~isnan(V1));  andel=length(F)/length(T)*100;
        mm=sprintf('%2.0f',andel);   text(length(L)/20,(1+yl(i)*3/4),mm,'color','r');
        ylabel(variables(i));
    elseif addormul == 3,
        yl=[1 1 1 1]; % Set the Y-limits
        set(gca,'ylim',[1-yl(i) 1+yl(i)]);
        plot(xlim,[1 1],'r-');
        F=find((abs(V1-1)-E1(I)) < 0);  T=find(~isnan(V1));  andel=length(F)/length(T)*100;
        mm=sprintf('%2.0f',andel);   text(length(L)/20,(1+yl(i)*3/4),mm,'color','r');
        ylabel(variables(i));
    end
    
    
    % Make a header to know which region
    if i==1;
       text(1 , (max(ylim)-min(ylim))*0.3+max(ylim) ,region,'fontsize',14,'fontweight','bold');
    end

   % Plots for SLSQ
   subplot(va,4,(i-1)*4+2);
   C1=SLSQcorr(:,i);
   errorbar(C1(I),E1(I),'k.-'); hold on
   plot(C1(I),'.r');
   L=find(~isnan(V1)); 
   if isempty(L),
        set(gca,'xlim',[0 1]);
    else
        set(gca,'xlim',[0 length(L)]);
    end
   if i == 1,
       title('SLSQ');
   end
   if addormul == 1,
       set(gca,'ylim',[-yl(i) yl(i)]); 
       plot(xlim,[0 0],'r-');
       F=find((abs(C1)-E1) < 0);  T=find(~isnan(C1));  andel=length(F)/length(T)*100;
       mm=sprintf('%2.0f',andel);   text(length(L)/20,yl(i)*7/8,mm,'color','r');
   else
       set(gca,'ylim',[1-yl(i) 1+yl(i)]);
       plot(xlim,[1 1],'r-');
       F=find((abs(C1-1)-E1) < 0);  T=find(~isnan(C1));  andel=length(F)/length(T)*100;
       mm=sprintf('%2.0f',andel);   text(length(L)/20,(1+yl(i)*3/4),mm,'color','r');
   end
         
   % Plots for WLSQ
   subplot(va,4,(i-1)*4+3);
   C2=WLSQcorr(:,i);
   errorbar(C2(I),E1(I),'k.-'); hold on
   plot(C2(I),'.r');
   L=find(~isnan(V1)); 
   if isempty(L),
        set(gca,'xlim',[0 1]);
    else
        set(gca,'xlim',[0 length(L)]);
    end
   if i == 1,
       title('WLSQ');
   end
   if addormul == 1,
       set(gca,'ylim',[-yl(i) yl(i)]); 
       plot(xlim,[0 0],'r-');
       F=find((abs(C2)-E1) < 0);  T=find(~isnan(C2));  andel=length(F)/length(T)*100;
       mm=sprintf('%2.0f',andel);   text(length(L)/20,yl(i)*7/8,mm,'color','r');
   else
        set(gca,'ylim',[1-yl(i) 1+yl(i)]);
        plot(xlim,[1 1],'r-');
        F=find((abs(C2-1)-E1) < 0);  T=find(~isnan(C2));  andel=length(F)/length(T)*100;
        mm=sprintf('%2.0f',andel);   text(length(L)/20,(1+yl(i)*3/4),mm,'color','r');
   end
   
   % Plots for WDLSQ
   subplot(va,4,(i-1)*4+4);
   C3=WDLSQcorr(:,i);
   errorbar(C3(I),E1(I),'k.-'); hold on
   plot(C3(I),'.r');
   L=find(~isnan(V1));
   if isempty(L),
        set(gca,'xlim',[0 1]);
    else
        set(gca,'xlim',[0 length(L)]);
    end
   if i == 1,
       title('WDLSQ');
   end
   if addormul == 1,
       set(gca,'ylim',[-yl(i) yl(i)]); 
       plot(xlim,[0 0],'r-');
       F=find((abs(C3)-E1) < 0);  T=find(~isnan(C3));  andel=length(F)/length(T)*100;
       mm=sprintf('%2.0f',andel);   text(length(L)/20,yl(i)*7/8,mm,'color','r');
   else
        set(gca,'ylim',[1-yl(i) 1+yl(i)]);
        plot(xlim,[1 1],'r-');
        F=find((abs(C3-1)-E1) < 0);  T=find(~isnan(C3));  andel=length(F)/length(T)*100;
        mm=sprintf('%2.0f',andel);   text(length(L)/20,(1+yl(i)*3/4),mm,'color','r');
   end
end
   
orient landscape
print -depsc Solution

% Change here if you want to plot another adjustment!
% Make a plot of the suggested adjustments and its standard deviation
posfig(2);
if addormul==2,
    cov=covWDLSQ;
    adj=WDLSQ+1;
else
    cov=covWDLSQ;  
    adj=WDLSQ;
end

% Add lines for those cruises that didn't have any Xovers (to avoid
% confusion having double numbering of the cruises)
new=NaN*ones(max(uc),length(variables));  
for i=1:length(uc)
    new(uc(i),:)=adj(i,:);
end
adj=new;   
for i=1:length(uc)
    new(uc(i),:)=cov(i,:);
end
cov=new;  
% make a vector of the station numbers
statnum=(1:1:length(adj))';

% =======================================
% Plot the adjustments
for i=1:va, % no variables
    subplot(va,1,i);
    adjustment=adj(:,i); covariance=cov(:,i);
    errorbar(adjustment,covariance,'k.'); hold on
    mm=sprintf(variables{i}); title(mm);
    ylabel('Adjustment');
    set(gca,'xlim',[0 length(adj)]);
    grid on
    if addormul == 1,
        yl=[50 20 20 0.05];
        set(gca,'ylim',[-yl(i) yl(i)]); 
        plot(xlim,[0 0],'r-');
        % to save the results for other plots
         cr=sprintf('add_%d  covariance adjustment statnum',i);
         eval(['save  ' cr]);
    else
        yl=[0.2 0.2 0.3 0.1 0.5];
        set(gca,'ylim',[1-yl(i) 1+yl(i)]);
        plot(xlim,[1 1],'r-');
        % to save the results for other plots
        cr=sprintf('mul_%d  covariance adjustment statnum',i);
        eval(['save  ' cr]);
    end
    if i == 1,
        diff(ylim)*0.2+max(ylim)
        text(1 , diff(ylim)*0.3+max(ylim) ,region,'fontsize',14,'fontweight','bold');
    end
end
xlabel('cruise #');  
 
orient tall
print -depsc Corrections
   
% Print the results to a file
  corrections=[adj , cov, statnum];
  F=find(isnan(corrections)); corrections(F)=-999;
  if addormul ==1,
      fid=fopen('corrections.txt' ,'w');
      fprintf(fid,'%4.2f %4.2f %4.2f %4.4f %4.2f %4.2f %4.2f %4.4f %4.0f\n',corrections');
  else
      fid=fopen('corrections.txt' ,'w');
      fprintf(fid,'%4.4f %4.4f %4.4f %4.4f %4.4f %4.4f %4.4f %4.4f %4.0f\n',corrections');
  end
  fclose(fid);
 
% produce a table with the sum of errors
OR=orginal;     F=find(isnan(OR)); OR(F)=0;
SL1=SLSQcorr;   F=find(isnan(SL1)); SL1(F)=0;
SL2=WLSQcorr;   F=find(isnan(SL2)); SL2(F)=0;
SL3=WDLSQcorr;  F=find(isnan(SL3)); SL3(F)=0;

sumerror=[sum(OR.^2).^0.5 ; sum(SL1.^2).^0.5 ; sum(SL2.^2).^0.5 ; sum(SL3.^2).^0.5]
stdall=[std(OR) ; std(SL1) ; std(SL2) ; std(SL3)]   
