function [diff, stdw] = xover_RC(cruise1,cruise2,parameter);

% Function to perform cross-over analysis of hydrographic data 
% INPUT:      CRUISE1  The ID of the first cruise in the x_over
%             CRUISE2  The ID of the second cruise in the x_over
%             PARAMETER 1:tco2; 2:alk; 3:oxygen; 4: nitrate; 5:silicate;
%                       6:phosphate; 7:salinity
%              
% OUTPUT:     DIFF     The weighted diffeence between the cruises
%             STDW     The weighted standard deviation of the comparison
%             A  figure will be generated in a new directory
%             with the name of the parameter.
%             
% The profiles are interpolated using Piecewise Cubic Hermite Interpolating, and then averaged to a average profile 
% for each cruise. Note that for less than 3 data for a depth/density no further analysis will be performed
% You will thus need at least 3 stations per cruise to do a X-over analyis.
% 
% To change the maximum distance between stations or set different min depth for -over analysis  
% you will have to edit the user defined seting in the file. 
%
% You will need to have m_map to run this function, and in addition the following files:
%
% Toste Tanhua  2009.06.05
% 

  diff=NaN;  stdw=NaN;
  cruise1
  cruise2
  
 %%  =========== USER DEFINED SETTINGS ===========
 
 % Define how many degrees of latitude to count as a X-over
 latlim=2; 
  % The minimum depth where the comparison should be made (default = 1500)
 mindepth=1500;
 
 
 % if addormul == 1; additive offsets
 % if addormul == 2; multiplicative offsets
 % SET THE APPROPRIATE LIMITS FOR THE PLOTTING ROUTINES!!
 if parameter ==1
     xmin=2100; xmax=2200; diffmin=-20; diffmax=20;     % tco2
      param=sprintf('tco2'); mkdir(param);
      addormul=1; ordin=1;
 elseif parameter ==2,
     xmin=2280; xmax=2380; diffmin=-15; diffmax=10;     % alk
      param=sprintf('alk'); mkdir(param);
      addormul=1; ordin=1;
 elseif parameter == 3,
     xmin=200; xmax=320;     diffmin=0.8; diffmax=1.2;    % oxygen
      param=sprintf('oxygen'); mkdir(param);
      addormul=2; ordin=1;
 elseif parameter == 4, 
     xmin=0; xmax=30;      diffmin=0.8; diffmax=1.2;    % nitrate
      param=sprintf('nitrate'); mkdir(param);
      addormul=2; ordin=1;
 elseif parameter ==5,
     xmin=0; xmax=2;       diffmin=0.8; diffmax=1.2;    % phosphate
      param=sprintf('phosphate'); mkdir(param);
      addormul=2; ordin=1;
 elseif parameter == 6,
     xmin=0; xmax=60;      diffmin=0.8; diffmax=1.2;    % silicate
      param=sprintf('silicate'); mkdir(param);
      addormul=2; ordin=1;
 elseif parameter == 7,
     xmin=34.6; xmax=36;     diffmin=-0.05; diffmax=0.05; % salinity (or ctdSal)
     param=sprintf('salinity'); mkdir(param);
     addormul=1; ordin=2;
 elseif parameter == 8,
     xmin=7.4; xmax=8.3;     diffmin=-0.05; diffmax=0.05; % pH (phsws25)
     param=sprintf('phsws25'); mkdir(param);
     addormul=1; ordin=1;
 end
 
  
      
 % =============== END OF USER DEFINED SETTINGS ========================
 
 global stat1 press1 dens1 param1 lon1 lat1 depth1 stat1 stat2 lat2 lon2 depth2 dens2 param2 expo1 expo2

 
   cruises=cruise1; stations=-999; load_carina; eval(['param1=' param ';']); 
   F=find(depth>mindepth & ~isnan(param1));
   lon1=longitude(F); lat1=latitude(F);  dens1=sigma4(F); param1=param1(F);
   depth1=depth(F); expo1=expocode; statnum1=station(F);
   [stat1,I,J] = unique(statnum1); lon1=lon1(I); lat1=lat1(I);
   
   cruises=cruise2; stations=-999; load_carina; eval(['param2=' param ';']); 
   F=find(depth>mindepth & ~isnan(param2));
   lon2=longitude(F); lat2=latitude(F); dens2=sigma4(F); param2=param2(F);
   depth2=depth(F); expo2=expocode; statnum2=station(F);
   [stat2,I,J] = unique(statnum2); lon2=lon2(I); lat2=lat2(I); 
   
   plotname=cat(2,param,'\',expo1{1},'___',expo2{1},'___',param,'___Xover');
   
          
   if length(param1) <= 2
           disp('Not enough deep data from cruise 1');
           cruise1
           disp(expo1)
           cruise2
           disp(expo2)
           diff=NaN; stdw=NaN;
           return
    end
   
   if length(param2) <= 2
       disp('Not enough deep data from cruise 2');
       cruise1
       disp(expo1)
       cruise2
       disp(expo2)
       diff=NaN; stdw=NaN;
       return
   end
   
   % Find crossovers and draw a map
  [STAT1,STAT2]=xovermap(latlim,mindepth);
  
  F=find(ismember(statnum1,STAT1));
  param1=param1(F); dens1=dens1(F); depth1=depth1(F); statnum1=statnum1(F);
    
  F=find(ismember(statnum2,STAT2));
  param2=param2(F); dens2=dens2(F); depth2=depth2(F); statnum2=statnum2(F);
     
  if length(param1) <= 2
           disp('Not enough deep data from cruise 1');
           cruise1
           disp(expo1)
           cruise2
           disp(expo2)
           diff=NaN; stdw=NaN;
           return
    end
   
   if length(param2) <= 2
       disp('Not enough deep data from cruise 2');
       cruise1
       disp(expo1)
       cruise2
       disp(expo2)
       diff=NaN; stdw=NaN;
       return
   end
   
   % Do the X-over vs. sigma4
    if ordin == 1,
        % Find the deepest station from cruise1 and get the density profile
        % interpolated to standard depths, and use this as "intdens"
        MD=find(depth1 == max(depth1)); DS=find(statnum1 == statnum1(MD(1)));
        imax=floor(max(depth1(DS)/10))*10;  imin=ceil(min(depth1(DS)/10))*10;  intd=[imin:20:imax];
        DDD=[min(dens1) ; dens1(DS)];   DEP=[mindepth ; depth1(DS)];
        % Make sure this profile is monotonous, i.e. no densities nor depth
        % should be there twice.
        if length(unique(DEP)) < length(DEP),
            DEP=unique(DEP); FF=find(unique(DEP));
            DDD=DDD(FF);
        end
        if length(unique(DDD)) < length(DDD),
            DDD=unique(DDD); FF=find(unique(DDD));
            DEP=DEP(FF);
        end
        
        out=intprofile(DDD,DEP,DEP,intd);
        intdens=sort(out(:,1));
        imax=(floor(max(intdens*1000))/1000)+0.002;
        imin=(ceil(min(intdens*1000))/1000)-0.02;
        % If the highest density is not in intdens, add it (i.e. if the deepest station is not the densest one).
        if max(intdens)+0.05 < max(dens1),
            intdens=[intdens; (max(intdens+0.05):0.05:max(dens1))'];
        end
        % If the lowest density is not in indense, add it
        if min(intdens)-0.05 > min(dens1),
             intdens=[(min(dens1):0.05:min(intdens-0.05))' ; intdens];
        end
        
        intdens=intdens';
        if isnan(intdens),
            imax=(floor(max(dens1*1000))/1000)-0.002; imin=ceil(min(dens1*1000))/1000;
            intdens=[44:0.01:45.7 45.705:0.0025:45.88 45.881:0.001:46];
            F=find(intdens>imin & intdens<imax); intdens=intdens(F);
        end
        if length(intdens)<2,
           disp('Not enough deep data from cruise 1 in the x-over area');
           cruise1
           disp(expo1)
           cruise2
           disp(expo2)
           diff=NaN; stdw=NaN;
           return
        end    
  end

  
 % Do the X-over vs. depth
    if ordin == 2,
        imax=floor(max(depth1/10))*10;  imin=ceil(min(depth1/10))*10;
        intdens=[imin:20:imax];
        if length(intdens)<2,
           disp('Not enough deep data from cruise 1 in the x-over area');
           cruise1
           disp(expo1)
           cruise2
           disp(expo2)
           diff=NaN; stdw=NaN;
           return
        end    
        dens1=depth1; dens2=depth2;
        imax=imax+100;
    end
    
   %===========================================================
   
   % Plot the profiles vs. density or depth
   % Positions [left, bottom, width, height]
   htop=axes('position',[.1 .07 .4 .55]);
   plot(param1,dens1,'.b','markersize',12);  hold on;  plot(param2,dens2,'xr');
   set(gca,'ydir','rev'); set(gca,'box','off'); set(gca,'xaxislocation','top'); 
   set(gca,'xlim',[xmin xmax]); set(gca,'ylim',[imin imax]);
   if ordin == 1,
       ylabel('Sigma-4','fontsize',14);
   else
       ylabel('Pressure  [db]','fontsize',14);
   end
   title(param,'fontsize',18); 
   
    % Add legend
   legend(expo1{1},expo2{1});
   set(legend,'box','off'); set(legend,'position',[0.1 0.1 0.2 0.2]);
   set(legend,'visible','off'); set(legend,'fontsize',12);
   
   % Create profiles interpolated to standard densities with their standard deviations
   % These are only for plotting purposes and are not used in the
   % calculation of the offsets.
   [MPROF]=meanprofile(statnum1,dens1,param1,depth1,intdens);
   if isempty(MPROF),
       disp('Not enough data from cruise1 to do Xover');
       disp('try increasing latlim, or check that there is data for cruise1');
       disp('          ');
       disp(expo1)
       disp(expo2)
       orient tall;  
       eval(['print -dpng -r300  ' plotname]);
       diff=NaN; stdw=NaN;
       return
   end
   D1=MPROF(:,1); P1=MPROF(:,2); S1=MPROF(:,3);
   plot(P1,D1,'b-'); hold on
 
   [MPROF]=meanprofile(statnum2,dens2,param2,depth2,intdens);
   if isempty(MPROF),
       disp('Not enough data from cruise2 to do Xover');
       disp('try increasing latlim, or check that there is data for cruise2');
       disp('          ');
       disp(expo1)
       disp(expo2)
       orient tall;  
       eval(['print -dpng -r300  ' plotname]);
       diff=NaN; stdw=NaN;
       return
   end
   D2=MPROF(:,1); P2=MPROF(:,2); S2=MPROF(:,3);
   plot(P2,D2,'r-');
    
   %===============================================================
   % Calculate the offset for each pair of stations within the influence radius
   RELOFF=[];
   for k=1:length(STAT1),
       F=find(statnum1 == STAT1(k)); 
       out1=intprofile(param1(F),dens1(F),depth1(F),intdens);
       F=find(stat1 == STAT1(k)); lat=mean(lat1(F)); lon=mean(lon1(F));
       lonlim=latlim / cos(mean(lat1(F))/180*pi);
              
       % Find all stations in cruise 2 that are within the lat and lon range of STAT1(i)
       K=find(lat2>lat-latlim & lat2<lat+latlim & lon2>lon-lonlim & lon2<lon+lonlim);
       statnum2a=stat2(K); USTAT2=unique(statnum2a);
       % calculate the difference between each pair of stations from STAT1(k)
       RO=[];
       for l=1:length(USTAT2),
           M=find(statnum2 == USTAT2(l));
           out2=intprofile(param2(M),dens2(M),depth2(M),intdens);
           [x1,x2,y]=matching(out1(:,1),out2(:,1),out1(:,2),out2(:,2)); 
           
           if addormul==1,
              % Calculate the additative offset 
               reloff=[x1-x2 , y];
               RO=[RO;reloff];
           end
           
           if addormul==2,
              % Calculate the multplicative offset 
              reloff=[(x1./x2) , y];
              RO=[RO;reloff];
           end
       end
       RELOFF=[RELOFF;RO];
   end
   
   if isempty(RELOFF)
       disp('Not enough data from cruise2 to do Xover');
       disp('try increasing latlim, or check that there is data for cruise2');
       disp('          ');
       disp(expo1)
       disp(expo2)
       orient tall;  
       eval(['print -dpng -r300  ' plotname]);
       diff=NaN; stdw=NaN;
       return
   end
   Y=RELOFF(:,2);  X=RELOFF(:,1);
      
   % Do a mean difference profile and plot
   PARmean=[]; PARstd=[];
   ude=unique(Y);
   for i=1:length(ude),
       F=find(Y==ude(i));
       Pm=nanmean(X(F));
       Ps=nanstd(X(F));
       % If less than 5 profiles, return NaN for standard deviation
       if length(F)< 5
          Ps=NaN;
       end
      PARmean=[PARmean;Pm];
      PARstd=[PARstd;Ps];
   end
 
  % Calculate the weighted difference
  F=find(~isnan(PARstd)); PARmean=PARmean(F); PARstd=PARstd(F); ude=ude(F);
  diff=nansum(PARmean./PARstd.^2)/nansum(1./PARstd.^2);
  stdw=nansum(1./PARstd)/nansum(1./PARstd.^2);
  %stdw=nanstd(RELOFF(:,1)); 
  
  if length(STAT1) < 3  | length(STAT2)<3 | length(PARmean)<1
     disp('Too few data to make a X-over');
     orient tall;  
     eval(['print -dpng -r300  ' plotname])
     diff=NaN; stdw=NaN;
     return
  end

  % Plot the differences 
  % Positions   [left, bottom, width, height]
  htop=axes('position',[.55 .07 .4 .55]);
  plot(PARmean,ude,'k.'); hold on;
  plot(PARmean+PARstd,ude,'k:'); plot(PARmean-PARstd,ude,'k:');
  set(gca,'ydir','rev'); set(gca,'box','off'); set(gca,'xaxislocation','top');
  set(gca,'ylim',[imin imax]); 
  set(gca,'xlim',[diffmin diffmax]);
  if addormul==1,
     plot([0 0],[0 10000],'k-'); 
  else
     plot([1 1],[0 10000],'k-');
  end
  plot([0 0],[0 10000],'k-');
  plot([diff diff],[min(intdens) max(dens1)+0.01],'r-','linewidth',2);
  plot([diff-stdw diff-stdw],[min(intdens) max(dens1)+0.01],'r:','linewidth',2);
  plot([diff+stdw diff+stdw],[min(intdens) max(dens1)+0.01],'r:','linewidth',2);
  if addormul == 1,
      mm=[expo1,' - ', expo2];
  else
      mm=[expo1,' / ', expo2];
  end
  title(mm,'fontsize',16); 
 
        
   % Write the results to the plot
   bla=(imax-imin)/100;  blu=diffmin+((diffmax-diffmin)/20);
   if addormul == 1,
       text(blu,imin-60*bla,'Additative offset','fontweight','bold');
   else
       text(blu,imin-60*bla,'Multiplicative offset','fontweight','bold');
   end
   
   distance=latlim*60*1.851; mm=sprintf('max distance [km] %2.0f',distance); text(blu,imin-55*bla,mm);
   mm=sprintf('weighted offset     %1.5f',diff); text(blu,imin-50*bla,mm);
   mm=sprintf('weighted stddev  %1.5f',stdw); text(blu,imin-45*bla,mm);
   mm=sprintf('# samples C1 (blue)     %2.0f',length(param1)); text(blu,imin-40*bla,mm);
   mm=sprintf('# samples C2 (red)       %2.0f',length(param2)); text(blu,imin-35*bla,mm);
   tt=length(STAT1); mm=sprintf('# stations C1 (blue)     %2.0f',tt); text(blu,imin-30*bla,mm);
   tt=length(STAT2); mm=sprintf('# stations C2 (red)      %2.0f',tt); text(blu,imin-25*bla,mm);
 
   orient tall
   eval(['print -dpng -r300  ' plotname])
  
